#############################################################################
# DAG-InstallRollup.ps1
# Handles tasks required to install update rollups on Exchange 2010 server
# that are part of a Database Availability Group (DAG)
#
# Pat Richard, MVP
# http://ucblogs.net/blogs/exchange
# 
# 1.0  Original script 12/16/2009
# 
# Dedicated blog post:
# http://www.ucblogs.net/blogs/exchange/archive/2009/12/15/Installing-Exchange-2010-rollups-on-DAG-servers.aspx
#
# Some info taken from
# http://blog.flaphead.dns2go.com/archive/2009/12/04/exchange-2010-training-day-5.aspx
# http://www.bhargavs.com/index.php/exchange-server/2009/12/how-do-i-check-update-rollup-version-on-exchange-20xx-server/
# http://blogs.technet.com/scottschnoll/archive/2009/12/10/installing-update-rollup-1-for-exchange-2010-on-dag-members.aspx
#############################################################################
$opt = "None"
clear
Do { 
	# clear
	if ($opt -ne "None") {write-host "Last command: "$opt -foregroundcolor Yellow}
	write-host
	write-host "Exchange Server 2010 - DAG rollup installation script"
	write-host "Please, select the step you would like to perform.."
	write-host
	write-host "1) Disable activation on this server"
	write-host "2) Switchover active databases to another server"
	write-host "3) Resume activation on this server"
	write-host "4) Correct activated databases"
	write-host "5) Show activated databases"
	write-host
	write-host "x) End"
	write-host
	$opt = Read-Host "Select an option.. [1-5,x]? "
	write-host

	switch ($opt)    {
		1	{Get-MailboxDatabaseCopyStatus Server (hostname) | Suspend-MailboxDatabaseCopy ActivationOnly Confirm:$false -SuspendComment "Installing update rollup for Exchange 2010"}
		2 {Move-ActiveMailboxDatabase -Server (hostname) -confirm:$false;write-host "Install the update rollup before continuing" -foregroundcolor yellow;exit}
		3 {Get-MailboxDatabaseCopyStatus Server (hostname) | Resume-MailboxDatabaseCopy }
		4 {Get-MailboxDatabase | Sort Name | % {$db=$_.Name; $xNow=$_.Server.Name ;$dbown=$_.ActivationPreference| Where {$_.Value -eq 1}; Write-Host $db "on" $xNow "Should be on" $dbOwn.Key -NoNewLine; If ( $xNow -ne $dbOwn.Key){Write-host " WRONG" -ForegroundColor Red; $blah = $db+"\"+$dbOwn.Key; if ((Get-MailboxDatabaseCopyStatus $blah).ActivationSuspended -eq $false){Move-ActiveMailboxDatabase $db -ActivateOnServer $dbOwn.Key -confirm:$False}else{write-host "Database can't be activated because activation is disabled for $blah" -foregroundcolor red}}ELSE{Write-Host " OK" -Foregroundcolor Green}}}
		5 {Get-MailboxDatabase | Sort Name | % {$db=$_.Name; $xNow=$_.Server.Name ;$dbown=$_.ActivationPreference| Where {$_.Value -eq 1}; Write-Host $db "on" $xNow "Should be on" $dbOwn.Key -NoNewLine; If ( $xNow -ne $dbOwn.Key){Write-host " WRONG" -ForegroundColor Red; }ELSE{Write-Host " OK" -Foregroundcolor Green}}}
		6 {
# Store header in variable
$headerLine = 
@"
 
Server Name,Rollup Update Description,Installed Date,ExSetup File Version
"@
 
# Write header to file
# $headerLine | Out-File .\results.csv -Encoding ASCII -Append
$headerLine
 
function getRU()
{
# Set server to connect to
	$Server = "$_".ToUpper()
 
 
# Check if server is running Exchange 2007 or Exchange 2010
 
	$ExchVer = (Get-ExchangeServer $Server | ForEach {$_.AdminDisplayVersion.Major})
 
# Set appropriate base path to read Registry
# Exit function if server is not running Exchange 2007 or Exchange 2010
	if ($ExchVer -eq "8" -or $ExchVer -eq "14")
		{
			switch ($ExchVer)
			{
			 "14"	{
			 			$REG_KEY = "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Installer\\UserData\\S-1-5-18\\Products\\AE1D439464EB1B8488741FFA028E291C\\Patches"
						$Reg_ExSetup = "SOFTWARE\\Microsoft\\ExchangeServer\\v14\\Setup"
			 		}
			 "8"	{
			 			$REG_KEY = "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Installer\\UserData\\S-1-5-18\\Products\\461C2B4266EDEF444B864AD6D9E5B613\\Patches"
						$Reg_ExSetup = "SOFTWARE\\Microsoft\\Exchange\\Setup"
			 		}
			}
		} 
	else
		{return}
 
# Read Rollup Update information from servers
 
# Set Registry constants
	$VALUE1 = "DisplayName"
	$VALUE2 = "Installed"
	$VALUE3 = "MsiInstallPath"
 
# Open remote registry
	$reg = [Microsoft.Win32.RegistryKey]::OpenRemoteBaseKey('LocalMachine', $Server)
 
# Set regKey for MsiInstallPath
	$regKey= $reg.OpenSubKey($REG_ExSetup)
 
# Get Install Path from Registry and replace : with $
	$installPath = ($regkey.getvalue($VALUE3) | foreach {$_ -replace (":","`$")})
 
# Set ExSetup.exe path
	$binFile = "Bin\ExSetup.exe"
 
# Get ExSetup.exe file version
	$exSetupVer = ((Get-Command "\\$Server\$installPath$binFile").FileVersionInfo | ForEach {$_.FileVersion})
 
# Create an array of patch subkeys
	$regKey= $reg.OpenSubKey($REG_KEY).GetSubKeyNames() | ForEach {"$Reg_Key\\$_"}
 
# Walk through patch subkeys and store Rollup Update Description and Installed Date in array variables
	$dispName = [array] ($regkey | %{$reg.OpenSubKey($_).getvalue($VALUE1)})
	$instDate = [array] ($regkey | %{$reg.OpenSubKey($_).getvalue($VALUE2)})
 
# Loop Through array variables and output to a file
	$countmembers = 0
 
	if ($regkey -ne $null)
	{
		while ($countmembers -lt $dispName.Count)
		{
		# $server+","+$dispName[$countmembers]+","+$instDate[$countmembers].substring(0,4)+"/"+$instDate[$countmembers].substring(4,2)+"/"+$instDate[$countmembers].substring(6,2)+","+$exsetupver | Out-File .\results.csv -Encoding ASCII -Append
		$server+","+$dispName[$countmembers]+","+$instDate[$countmembers].substring(4,2)+"/"+$instDate[$countmembers].substring(6,2)+"/"+$instDate[$countmembers].substring(0,4)+","+$exsetupver
		$countmembers++
		}
	}
	else
	{
		# $server+",No Rollup Updates are installed,,"+$exsetupver | Out-File .\results.csv -Encoding ASCII -Append
		$server+",No Rollup Updates are installed,,"+$exsetupver
	}
}
 
# Get Exchange 2007 servers and write Rollup Updates to results file
$Servers = (Get-ExchangeServer | Where-Object {($_.AdminDisplayVersion -match "8" -OR $_.AdminDisplayVersion -match "14") -AND $_.ServerRole -ne "ProvisionedServer" -and $_.ServerRole -ne "Edge"} | ForEach {$_.Name})
$Servers | ForEach {getRU}
write-host		
		}
		x {exit}
		default {}
	}
}
while ($opt -ne "x")